{% load i18n %}<!DOCTYPE html>
<html lang="en">
<head>
    <meta http-equiv="content-type" content="text/html; charset=utf-8">
    <title>{% translate "SSO token validation error" %}</title>
    <meta name="robots" content="NONE,NOARCHIVE">
    <style>
        html * {
            padding: 0;
            margin: 0;
        }

        body * {
            padding: 10px 20px;
        }

        body * * {
            padding: 0;
        }

        body {
            font: small sans-serif;
            background: #eee;
            color: #000;
        }

        body > div {
            border-bottom: 1px solid #ddd;
        }

        h1 {
            font-weight: normal;
            margin-bottom: .4em;
        }

        h1 span {
            font-size: 60%;
            color: #666;
            font-weight: normal;
        }

        table {
            border: none;
            border-collapse: collapse;
            width: 100%;
        }

        td, th {
            vertical-align: top;
            padding: 2px 3px;
        }

        th {
            width: 12em;
            text-align: right;
            color: #666;
            padding-right: .5em;
        }

        #info {
            background: #f6f6f6;
        }

        #info ol {
            margin: 0.5em 4em;
        }

        #info ol li {
            font-family: monospace;
        }

        #summary {
            background: #ffc;
        }

        #explanation {
            background: #eee;
            border-bottom: 0px none;
        }

        pre.exception_value {
            font-family: sans-serif;
            color: #575757;
            font-size: 1.5em;
            margin: 10px 0 10px 0;
        }
    </style>
</head>
<body>
<div id="summary">
    <h1>{% translate "SSO token validation error" %}</h1>
</div>
<div id="info">
    <pre style="margin-left: 30px; font-weight: bold; white-space: pre-wrap">{{ error }}</pre>
</div>
<div id="explanation">
    <p>This error caused because SSO server don't know token of this service.</p>
    <p>
        To solve this problem create subordinated service in SSO-server and
        put obtained token to SSO[TOKEN] into settings.py file
    </p>
    <p>
        You’re seeing this error because you have <code>DEBUG = True</code>
    </p>
</div>
</body>
</html>