{% load i18n %}<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="UTF-8">
        <meta http-equiv="refresh" content="0; url='{{ redirect_url }}'">
    </head>
    <body>
        <a href="{{ redirect_url }}">{% translate "Click here to continue authorization" %}</a>
    </body>
</html>